#FOR INSTRUCTIONS, SEE LINE 101!!
#
#NOTE:
#The excel version used for the practicals has minor differences in computed values 
#
#

#Tested with R version 4.3
location = "C://ProgramData/R/win-library/4.3"

#download and install required R packages if needed
#install.packages('deSolve', lib.loc = location)
#install.packages('this.path', lib.loc = location)
require('this.path', lib.loc = location)

#Get path of current script
scriptPath <- here()
#Set working directory to script path
setwd(scriptPath)

source('./Components/LINTUL_get_weather.r')	
source('./Components/LINTUL_parameters.r')	
source('./Components/lINTUL1_model.r')	
require('deSolve', lib.loc = location)  #used for solving ODEs

#===================================================================================
#HELP FUNCTIONS
#===================================================================================
#Help function to assist in running LINTUL1.
RUN_LINTUL1 <- function(YR, parms, result_filename, adjustLUE = TRUE){
  #Read in weather data
  wdata_measured <- get_weather(directory="./weather/" ,
                                country="NLD" ,
                                station=1,
                                year=substr(YR,2,4))
  #Default: add CO2 adjusted LUE to parameter list
  if(adjustLUE){
    parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = YR, NRMONTH = 6, Silent = TRUE)
  }
  
  #Run the LINTUL1 model
  results <- ode(y = LINTUL1_iniSTATES(parms), 
                 time = seq(STTIME, FINTIM, by = 1), 
                 func = LINTUL1, 
                 parms = parms,
                 WDATA = wdata_measured,
                 method = "euler")
  #Convert to a data.frame
  results <- as.data.frame(results)
  #Write to file
  write.csv(data.frame(results),file=result_filename)
  return(results)
}

#Define figure to compare scenario's
#Includes 4 panels and these show: BIOMASS, YIELD, LAI and HI
FigureA <- function(Title = year, 
                    list_results, 
                    LegendText,
                    cols=c('black','red','blue','green','orange','cyan','purple','yellow'),
                    ltys=c(      1,    2,     4,      1,       2,     4,       1,       2),
                    lwds=c(      1,  1.5,     2,      1,     1.5,     2,       1,     1.5)){
  
  par(mfrow=c(2,2),mar=c(2,4,1,1))
  plot(list_results[[1]][,"time"], list_results[[1]][,"WBIOMTHA"],  
       ylab ="Total biomass, t DM/ha",xlim=c(30,300),ylim=c(0,25),main=Title, type="l",
       col=cols[1],lwd=lwds[1], lty=ltys[1])
  for(i in 2:length(list_results)){
    lines(list_results[[i]][,"time"], list_results[[i]][,"WBIOMTHA"],  
       col=cols[i],lwd=lwds[i], lty=ltys[i])
  }

  plot(list_results[[1]][,"time"], list_results[[1]][,"WSOTHA"],  
       ylab ="Grain yield, t DM/ha",xlim=c(30,300),ylim=c(0,10),main=Title, type="l",
       col=cols[1],lwd=lwds[1], lty=ltys[1])
  for(i in 2:length(list_results)){
    lines(list_results[[i]][,"time"], list_results[[i]][,"WSOTHA"],  
          col=cols[i],lwd=lwds[i], lty=ltys[i])
  } 
  
  plot(list_results[[1]][,"time"], list_results[[1]][,"LAI"],  
       ylab ="LAI, m2 leaf/ m2 soil",xlim=c(30,300),ylim=c(0,6),type="l",
       col=cols[1],lwd=lwds[1], lty=ltys[1])
  for(i in 2:length(list_results)){
    lines(list_results[[i]][,"time"], list_results[[i]][,"LAI"],  
          col=cols[i],lwd=lwds[i], lty=ltys[i])
  }
  
  plot(list_results[[1]][,"time"], list_results[[1]][,"HI"],  
       ylab ="Harvest index",xlim=c(30,300),ylim=c(0,0.6), type="l",
       col=cols[1],lwd=lwds[1], lty=ltys[1])
  for(i in 2:length(list_results)){
    lines(list_results[[i]][,"time"], list_results[[i]][,"HI"],  
          col=cols[i],lwd=lwds[i], lty=ltys[i])
  }  
  legend("topleft",legend=LegendText,col = cols, lwd=lwds, lty=ltys, bty ="n")
}
#===================================================================================
#END HELP FUNCTIONS
#===================================================================================


#---------------------------------Run control---------------------------------------#
#                                                                                   #
#-----------------------------------------------------------------------------------#
#GENERAL SETTINGS
STTIME  <- 30        # d     :     start time of simulation
FINTIM  <- 300       # d     :     finish time 
#-----------------------------------------------------------------------------------#

#INSTRUCTIONS:
#In this practical, you will use the crop growth model LINTUL1.
#Default parameters are available by calling this function:
parms <- LINTUL1_PARAMETERS_SPRINGWHEAT()

#This provides the parameters in a list. 
#The RUE for the year of calculation needs to be added. 
#Let's say that the year is 2011 and you want to have the RUE for June:
#You can add the RUE for 2011 by calling this function:
parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = 2011, NRMONTH = 6, Silent = FALSE)

#Call the help function LINTUL1 to run the model. This will:
#1. Read in weather data for station 1 in the Netherlands for the year 2011
#2. Save the results to file in the Results folder with the given file name
#3. Returns a data-frame with calculated values per day
result_default <- RUN_LINTUL1(YR = 2011, 
                              parms = parms, 
                              result_filename = "./Results/LINTUL1_2011_defaults.csv")



#A2===========================================================================================================
YR = 2011
#LINTUL with irrigation: POTENTIAL PRODUCTION
result_default <- RUN_LINTUL1(YR = YR, 
                              parms = LINTUL1_PARAMETERS_SPRINGWHEAT(), 
                              result_filename = "./Results/LINTUL1_2011_defaults.csv")
#Get growth rate
result_default[,"RGROWTH"]<-rowSums(subset(result_default,select=c("RWRT", "RWLV", "RWST", "RWSO")))

#Find day with maximum growth rate
ii<- which(result_default[,"RGROWTH"] == max(result_default[,"RGROWTH"],na.rm = TRUE))
print("The day with the highest growth rate is:")
print(result_default[ii,c("time","RGROWTH", "LAI", "RPAR", "PARINT")])


#A3===========================================================================================================
df<-data.frame(scen = "Default",
           max_LAI = max(result_default[,"LAI"]),
           WBIOMTHA = max(result_default[,"WBIOMTHA"]),
           WSOTHA = max(result_default[,"WSOTHA"]))
df[,"FRESH_YIELD_THA"] <- df[,"WSOTHA"]/0.85
print(df)

#A4===========================================================================================================
parms <- LINTUL1_PARAMETERS_SPRINGWHEAT()
parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = YR, NRMONTH = 6, Silent = FALSE)

sla_default <- parms$SLA
RDRSHM_default <- parms$RDRSHM
parms$SLA <- 0.015
result_sla <- RUN_LINTUL1(YR = YR, parms = parms, result_filename = "./Results/LINTUL1_2011_SLA_0.015.csv")

parms$SLA <- sla_default
parms$RDRSHM <- 0.06
result_shm <- RUN_LINTUL1(YR = YR, parms = parms, result_filename = "./Results/LINTUL1_2011_RDRSHM_0.06.csv")

compare <- data.frame(scen = "Default",
                      max_LAI = max(result_default[,"LAI"]),
                      WBIOMTHA = result_default[nrow(result_default),"WBIOMTHA"],
                      WSOTHA = result_default[nrow(result_default),"WSOTHA"],
                      HI = result_default[nrow(result_default),"HI"])
compare <- rbind(compare,
                 data.frame(scen = "0.015SLA",
                            max_LAI = max(result_sla[,"LAI"]),
                            WBIOMTHA = result_sla[nrow(result_sla),"WBIOMTHA"],
                            WSOTHA = result_sla[nrow(result_sla),"WSOTHA"],
                            HI = result_sla[nrow(result_sla),"HI"]))
                 
compare <- rbind(compare,
                 data.frame(scen = "0.06RDRSHM",
                      max_LAI = max(result_shm[,"LAI"]),
                      WBIOMTHA = result_shm[nrow(result_shm),"WBIOMTHA"],
                      WSOTHA = result_shm[nrow(result_shm),"WSOTHA"],
                      HI = result_shm[nrow(result_shm),"HI"]))
print("ANSWER FOR A4")
print(compare)
print("END ANSWER")


#plot scenarios with selected variables
windows(xpos=-25)
FigureA(Title= YR,
        list_results = list(read.csv("./Results/LINTUL1_2011_defaults.csv"), 
                            read.csv("./Results/LINTUL1_2011_SLA_0.015.csv"), 
                            read.csv("./Results/LINTUL1_2011_RDRSHM_0.06.csv")),
        LegendText=c("Default","SLA 0.015","RDRSHM 0.06"))

#A5===========================================================================================================
parms <- LINTUL1_PARAMETERS_SPRINGWHEAT()
parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = YR, NRMONTH = 6, Silent = TRUE)
#Store default values
K_default <- parms$K
LUEREF_default <- parms$LUEREF
#Change parameter value for K
parms$K <- 0.8
result_K <- RUN_LINTUL1(YR = YR, parms = parms, result_filename = "./Results/LINTUL1_2011_0.8K.csv")
parms$K <- K_default

#Add as row to the dataframe
compare <- rbind(compare,
                 data.frame(scen = "0.8K",
                            max_LAI = max(result_K[,"LAI"]),
                            WBIOMTHA = result_K[nrow(result_K),"WBIOMTHA"],
                            WSOTHA = result_K[nrow(result_K),"WSOTHA"],
                            HI = result_K[nrow(result_K),"HI"]))


parms$LUEREF <- 2.5
#Ensure to also change to LUE value for the year
parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = YR, NRMONTH = 6, Silent = TRUE)
result_LUE <- RUN_LINTUL1(YR = YR, adjustLUE = FALSE, parms = parms,result_filename = "./Results/LINTUL1_2011_2.5RUE.csv")

compare <- rbind(compare,
                 data.frame(scen = "2.5RUE",
                            max_LAI = max(result_LUE[,"LAI"]),
                            WBIOMTHA = result_LUE[nrow(result_LUE),"WBIOMTHA"],
                            WSOTHA = result_LUE[nrow(result_LUE),"WSOTHA"],
                            HI = result_LUE[nrow(result_LUE),"HI"]))
print("ANSWER FOR A5")
print(compare)
print("END ANSWER")



#A8===========================================================================================================
parms <- LINTUL1_PARAMETERS_SPRINGWHEAT()
param_default <- parms

#Sensitivity analysis: see what -10% and +10% do to outcomes
for (param in c("K", "LUEREF", "TSUMAN")){
  for(adj in c(0.9, 1.1)){
    p_default <- 
    #Change parameter value
    parms[[param]] <- parms[[param]] * adj
    #Adjust TSUMS for partitioning when TSUMAN changes
    if(param == "TSUMAN"){
      parms[["FRTTB.TSUM"]] <- parms[["FRTTB.TSUM"]] * parms[[param]] / param_default[[param]]
      parms[["FSTTB.TSUM"]] <- parms[["FSTTB.TSUM"]] * parms[[param]] / param_default[[param]]
      parms[["FLVTB.TSUM"]] <- parms[["FLVTB.TSUM"]] * parms[[param]] / param_default[[param]]
      parms[["FSOTB.TSUM"]] <- parms[["FSOTB.TSUM"]] * parms[[param]] / param_default[[param]]
    }
    
    #Ensure that also LKUE is adapated accordingly
    parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = parms, YEAR = YR, NRMONTH = 6, Silent = TRUE)
        filename=paste("./Results/LINTUL1_2011_",toString(parms[param]),param,".csv")
    #Run the model with new parameter value
    result <- RUN_LINTUL1(YR = YR, parms = parms, result_filename =filename)
    #Add the result as a row to the compare data frame
    compare <- rbind(compare,
                     data.frame(scen = paste(toString(parms[param]),param),
                                max_LAI = max(result[,"LAI"]),
                                WBIOMTHA = result[nrow(result),"WBIOMTHA"],
                                WSOTHA = result[nrow(result),"WSOTHA"],
                                HI = result[nrow(result),"HI"]))
    #Change parameter values to default value
    parms <- param_default

  }
}
print("ANSWER FOR A8")
compare[,2:ncol(compare)] <- round(compare[,2:ncol(compare)], 2)
print(compare)
print("END ANSWER")


#B3===========================================================================================================

SCENARIOS <- NULL
#First calculate!!
for(YR in c(1972, 2011)){
  #Load weather data for selected year
  wdata_measured <- get_weather(directory="./weather/" ,
                                country="NLD" ,
                                station=1,
                                year=substr(YR,2,4))
  #Add a loop to evaluate effects of temperature
  for(TempInc in c(-2.0, -1.0, 0, 1.0, 1.5, 2.0, 4.0)){
    #A loop for PAR
    for(PARInc in c(0, 0.5, 1.0)){
      ScenName <- paste0(toString(YR),
                         " TempInc ", toString(TempInc),
                         " PARInc ", toString(PARInc) )
      #copy weather data
      wdata_plus <- wdata_measured
      #Change daily average temperature and PAR accordingly
      wdata_plus[,"DAVTMP"] <- wdata_plus[,"DAVTMP"] + TempInc
      wdata_plus[,"DTR"] <- wdata_plus[,"DTR"] + PARInc * 2
      
      #LINTUL with irrigation: POTENTIAL PRODUCTION
      #Add CO2 adjusted RUE to parameter list
      parms <- ADD_LUE_TO_PARAMETER_LIST (Pars = LINTUL1_PARAMETERS_SPRINGWHEAT(), 
                                          YEAR = YR, 
                                          NRMONTH = 6,
                                          Silent = TRUE)
      
      result <- ode(y = LINTUL1_iniSTATES(parms), 
                    time = seq(STTIME, FINTIM, by = 1), 
                    func = LINTUL1, 
                    parms = parms,
                    WDATA = wdata_plus,
                    method = "euler")
      #Store scenario results in a data frame
      SCENARIOS <- rbind(SCENARIOS, 
                         data.frame(Year = YR,
                                    TempInc = TempInc,
                                    PARinc = PARInc,
                                    Biomass_tha  = result[FINTIM - STTIME,"WBIOMTHA"],
                                    Yield_tha    = result[FINTIM - STTIME,"WSOTHA"],
                                    HarvestIndex = result[FINTIM - STTIME,"HI"]))
      
      #Write to file
      write.csv(data.frame(result),file=paste0("./Results/Run details for LINTUL1 for ",ScenName, ".csv",sep = "", collapse = NULL))
    }
  }
}
print("ANSWER FOR B3 and B4")
SCENARIOS[,2:ncol(SCENARIOS)] <- round(SCENARIOS[,2:ncol(SCENARIOS)], 2)
print(SCENARIOS)
print("END ANSWER")



#B5===========================================================================================================

scenarios = c("Current","+1.5dC","+1.5dC+1 MJ TDR/m2/d")
results_1972 = list(read.csv(paste0("./Results/Run details for LINTUL1 for 1972 TempInc 0 PARInc 0.csv",sep = "", collapse = NULL)), 
                    read.csv(paste0("./Results/Run details for LINTUL1 for 1972 TempInc 1.5 PARInc 0.csv",sep = "", collapse = NULL)), 
                    read.csv(paste0("./Results/Run details for LINTUL1 for 1972 TempInc 1.5 PARInc 0.5.csv",sep = "", collapse = NULL)))

#plot selected variables
windows(xpos=-25)
FigureA(Title= 1972,list_results = results_1972, LegendText=scenarios)

windows(xpos=-125)
results_2011 = list(read.csv(paste0("./Results/Run details for LINTUL1 for 2011 TempInc 0 PARInc 0.csv",sep = "", collapse = NULL)), 
                    read.csv(paste0("./Results/Run details for LINTUL1 for 2011 TempInc 1.5 PARInc 0.csv",sep = "", collapse = NULL)), 
                    read.csv(paste0("./Results/Run details for LINTUL1 for 2011 TempInc 1.5 PARInc 0.5.csv",sep = "", collapse = NULL)))
FigureA(Title= 2011, list_results = results_2011,LegendText=c("Current","+1.5dC","+1.5dC+1 MJ TDR/m2/d"))


